using System;
using System.Collections;
using UnityEngine;

public class BloodWireBehaviourNP : DistanceJointWireBehaviour
{
    private CirculationBehaviour limbA;
    private CirculationBehaviour limbB;
    private BloodContainer containerA;
    private BloodContainer containerB;
    private MaterialPropertyBlock materialPropertyBlock;
    private float flow;
    private Color color;
    private CardiopulmonaryBypassMachineBehaviour cardiopulmonaryBypass;
    private HeartMonitorBehaviour heartMonitor;
    private CirculationBehaviour heartLimb;
    public const float TransferRate = 20f;

    [SkipSerialisation]
    public bool CanTransferBlood => (bool)(UnityEngine.Object)this.containerA && (bool)(UnityEngine.Object)this.containerB && this.containerA.AllowsTransfer && this.containerB.AllowsTransfer;

    [SkipSerialisation]
    public bool AreBothLimbs { get; private set; }

    [SkipSerialisation]
    public bool IsHeartRateWire { get; private set; }

    [SkipSerialisation]
    public bool IsCardiopulmonaryBypassWire { get; private set; }

    protected override void Created()
    {
        base.Created();
        this.color = Color.clear;
        this.materialPropertyBlock = new MaterialPropertyBlock();
        this.lineRenderer.GetPropertyBlock(this.materialPropertyBlock);
        if (!(bool)(UnityEngine.Object)this.physicalBehaviour || !(bool)(UnityEngine.Object)this.otherPhysicalBehaviour || (UnityEngine.Object)this.physicalBehaviour == (UnityEngine.Object)this.otherPhysicalBehaviour)
        {
            this.AreBothLimbs = false;
            this.materialPropertyBlock.SetFloat(ShaderProperties.Get("_Flow"), 0.0f);
            this.materialPropertyBlock.SetColor(ShaderProperties.Get("_BloodColor"), Color.magenta);
            this.lineRenderer.SetPropertyBlock(this.materialPropertyBlock);
        }
        else
        {
            this.containerA = this.physicalBehaviour.GetComponent<BloodContainer>();
            this.containerB = this.otherPhysicalBehaviour.GetComponent<BloodContainer>();
            this.limbA = this.physicalBehaviour.GetComponent<CirculationBehaviour>();
            this.limbB = this.otherPhysicalBehaviour.GetComponent<CirculationBehaviour>();
            this.heartMonitor = this.physicalBehaviour.GetComponent<HeartMonitorBehaviour>();
            if (!(bool)(UnityEngine.Object)this.heartMonitor)
                this.heartMonitor = this.otherPhysicalBehaviour.GetComponent<HeartMonitorBehaviour>();
            this.cardiopulmonaryBypass = this.physicalBehaviour.GetComponent<CardiopulmonaryBypassMachineBehaviour>();
            if (!(bool)(UnityEngine.Object)this.cardiopulmonaryBypass)
                this.cardiopulmonaryBypass = this.otherPhysicalBehaviour.GetComponent<CardiopulmonaryBypassMachineBehaviour>();
            this.IsHeartRateWire = (bool)(UnityEngine.Object)this.heartMonitor;
            this.IsCardiopulmonaryBypassWire = (bool)(UnityEngine.Object)this.cardiopulmonaryBypass && ((bool)(UnityEngine.Object)this.limbA || (bool)(UnityEngine.Object)this.limbB) && ((bool)(UnityEngine.Object)this.limbA ? (this.limbA.WasInitiallyPumping ? 1 : 0) : (this.limbB.WasInitiallyPumping ? 1 : 0)) != 0 && ((bool)(UnityEngine.Object)this.limbA ? (this.limbA.Limb.IsAndroid ? 1 : 0) : (this.limbB.Limb.IsAndroid ? 1 : 0)) == 0;
            if (this.IsCardiopulmonaryBypassWire)
            {
                this.cardiopulmonaryBypass.PlayValidSound();
                ++this.cardiopulmonaryBypass.ValidConnectionCount;
                this.cardiopulmonaryBypass.UpdateStatusText();
            }
            else if ((bool)(UnityEngine.Object)this.cardiopulmonaryBypass)
            {
                this.cardiopulmonaryBypass.PlayInvalidSound();
                CirculationBehaviour circulationBehaviour = (bool)(UnityEngine.Object)this.limbA ? this.limbA : this.limbB;
                string s = string.Empty;
                if (!(bool)(UnityEngine.Object)circulationBehaviour)
                    s = "<color=red>INORGANIC</color>";
                else if (circulationBehaviour.Limb.IsAndroid)
                    s = "<color=red>ARTIFICIAL</color>";
                else if (!circulationBehaviour.WasInitiallyPumping)
                    s = "<color=red>NO HEART DETECTED</color>";
                if (!string.IsNullOrEmpty(s))
                    this.cardiopulmonaryBypass.StartCoroutine(flash());
                else
                    this.cardiopulmonaryBypass.UpdateStatusText();

                IEnumerator flash()
                {
                    this.cardiopulmonaryBypass.StatusText.text = s;
                    yield return (object)new WaitForSeconds(3f);
                    if ((bool)(UnityEngine.Object)this.cardiopulmonaryBypass)
                        this.cardiopulmonaryBypass.UpdateStatusText();
                }
            }
            if (this.IsHeartRateWire)
            {
                if ((bool)(UnityEngine.Object)this.limbA && !this.limbA.Limb.IsAndroid)
                    this.heartLimb = this.limbA;
                else if ((bool)(UnityEngine.Object)this.limbB && !this.limbB.Limb.IsAndroid)
                    this.heartLimb = this.limbB;
                else
                    this.IsHeartRateWire = false;
            }
            this.AreBothLimbs = (bool)(UnityEngine.Object)this.limbA && (bool)(UnityEngine.Object)this.limbB && !this.limbA.Limb.IsAndroid && !this.limbB.Limb.IsAndroid;
            this.materialPropertyBlock.SetFloat(ShaderProperties.Get("_Flow"), 0.0f);
            this.materialPropertyBlock.SetColor(ShaderProperties.Get("_BloodColor"), Color.black);
            this.lineRenderer.SetPropertyBlock(this.materialPropertyBlock);
        }
    }

    protected override void Update()
    {
        base.Update();
        this.ProcessCardiopulmonaryBypassWire();
    }

    private void ProcessCardiopulmonaryBypassWire()
    {
        if (!this.IsCardiopulmonaryBypassWire)
            return;
        float b = Mathf.Clamp01(this.cardiopulmonaryBypass.GetFinalIntensity());
        CirculationBehaviour circulationBehaviour = (bool)(UnityEngine.Object)this.limbA ? this.limbA : this.limbB;
        if (circulationBehaviour.Limb.NodeBehaviour.IsConnectedToRoot)
            circulationBehaviour.Limb.Person.OxygenLevel = Mathf.Max(0.5f * b, circulationBehaviour.Limb.Person.OxygenLevel);
        circulationBehaviour.ArtificialHeartbeat = this.cardiopulmonaryBypass.TargetHeartrate * b;
        circulationBehaviour.BloodFlow = Mathf.Max(circulationBehaviour.BloodFlow, b);
    }

    protected override void Tick()
    {
        if (this.IsHeartRateWire)
            this.SendHeartRate();
        float transferRate = Time.fixedDeltaTime * 2f;
        float fixedDeltaTime = Time.fixedDeltaTime;
        if (this.CanTransferBlood)
            this.Transfer(this.containerA, this.containerB, 1f, transferRate);
        BloodWireBehaviourNP.AveragePressure(fixedDeltaTime, this.containerA, this.containerB);
        if (this.AreBothLimbs)
        {
            float bloodFlow1 = this.limbA.BloodFlow;
            float bloodFlow2 = this.limbB.BloodFlow;
            float b = (float)(0.5 * ((double)bloodFlow1 + (double)bloodFlow2));
            if (this.limbA.AllowPressureTransfer && this.limbB.AllowPressureTransfer)
            {
                this.limbA.BloodFlow = Mathf.Lerp(bloodFlow1, b, fixedDeltaTime);
                this.limbB.BloodFlow = Mathf.Lerp(bloodFlow2, b, fixedDeltaTime);
            }
        }
        this.flow = Mathf.Lerp(this.flow, 0.0f, 0.1f);
        this.color = Color.Lerp(this.color, Color.clear, 0.1f);
    }

    //One way transfer

    private void Transfer(BloodContainer a, BloodContainer b, float visualDir, float transferRate)
    {
        this.TransferLiquid(transferRate, a, b, visualDir);
        /*if ((double)a.MeasuredPressure > (double)b.MeasuredPressure)
        {
            this.TransferLiquid(transferRate, a, b, visualDir);
        }
        else
        {
            if ((double)b.MeasuredPressure <= (double)a.MeasuredPressure)
                return;
            this.TransferLiquid(transferRate, b, a, -visualDir);
        }*/
    }

    private void SendHeartRate()
    {
        if (this.heartLimb.Limb.PhysicalBehaviour.isDisintegrated || this.physicalBehaviour.isDisintegrated)
        {
            this.heartMonitor.IsConnected = false;
        }
        else
        {
            this.heartMonitor.IsConnected = true;
            this.heartMonitor.ConnectedLimb = this.heartLimb.Limb;
        }
    }

    private void OnWillRenderObject()
    {
        this.materialPropertyBlock.SetFloat(ShaderProperties.Get("_Flow"), Mathf.Clamp(this.flow, -1f, 1f));
        this.materialPropertyBlock.SetColor(ShaderProperties.Get("_BloodColor"), this.color);
        this.lineRenderer.SetPropertyBlock(this.materialPropertyBlock);
    }

    private void TransferLiquid(
      float rate,
      BloodContainer from,
      BloodContainer to,
      float visualDirection)
    {
        if ((double)rate < 0.0)
            throw new Exception("Rate can't be negative");
        if ((double)from.TotalLiquidAmount <= (double)from.LowerLimit || (double)to.TotalLiquidAmount >= (double)to.UpperLimit || (double)Mathf.Abs(from.TransferTo(rate, to)) <= 1.40129846432482E-45)
            return;
        from.DeleteEmptyLiquidEntries();
        to.DeleteEmptyLiquidEntries();
        this.flow = Mathf.Lerp(this.flow, visualDirection, rate);
        this.color = Color.Lerp(this.color, from.GetComputedColor(), rate);
    }

    internal static void AveragePressure(float rate, BloodContainer from, BloodContainer to)
    {
        if (!(bool)(UnityEngine.Object)from || !(bool)(UnityEngine.Object)to)
            return;
        if ((double)rate < 0.0)
            throw new Exception("Rate can't be negative");
        float measuredPressure1 = from.MeasuredPressure;
        float measuredPressure2 = to.MeasuredPressure;
        float b = (float)(0.5 * ((double)measuredPressure1 + (double)measuredPressure2));
        if (!from.AllowPressureTransfer || !to.AllowPressureTransfer)
            return;
        from.MeasuredPressure = Mathf.Lerp(measuredPressure1, b, rate);
        to.MeasuredPressure = Mathf.Lerp(measuredPressure2, b, rate);
    }

    protected override void OnDestroy()
    {
        base.OnDestroy();
        if (this.IsHeartRateWire && (bool)(UnityEngine.Object)this.heartMonitor)
            this.heartMonitor.IsConnected = false;
        if (!this.IsCardiopulmonaryBypassWire || !(bool)(UnityEngine.Object)this.cardiopulmonaryBypass)
            return;
        ((bool)(UnityEngine.Object)this.limbA ? this.limbA : this.limbB).ArtificialHeartbeat = 0.0f;
        --this.cardiopulmonaryBypass.ValidConnectionCount;
        this.cardiopulmonaryBypass.UpdateActivation();
    }
}
